using System;
using System.IO;
using ChartGettingStarted;
using Foundation;
using QuickLook;
using SimpleSample;
using Syncfusion.Pdf;
using Syncfusion.Pdf.Graphics;
using Syncfusion.SfChart.iOS;
using Syncfusion.SfChart.XForms;
using Syncfusion.SfChart.XForms.iOS.Renderers;
using UIKit;
using Xamarin.Forms;

[assembly: Dependency(typeof(ChartGettingStarted.iOS.ChartDependencyService))]
namespace ChartGettingStarted.iOS
{
    public class ChartDependencyService : IChartDependencyService
    {
        public void SaveAsPDF(string filename, MemoryStream stream, SfChart formChart)
        {
            var doc = new PdfDocument();
            doc.PageSettings.Margins.All = 0;
            var page = doc.Pages.Add();
            var g = page.Graphics;

            var nativechart = SfChartRenderer.GetNativeObject(typeof(SfChart), formChart) as SFChart;

            UIGraphics.BeginImageContext(nativechart.Frame.Size);
            nativechart.Layer.RenderInContext(UIGraphics.GetCurrentContext());

            var image = UIGraphics.GetImageFromCurrentImageContext();
            UIGraphics.EndImageContext();

            var imgStream = image.AsJPEG().AsStream();

            g.DrawImage(PdfImage.FromStream(imgStream), 0, 0, 500, 700);

            doc.Save(stream);
            doc.Close(true);

            var path = Environment.GetFolderPath(Environment.SpecialFolder.Personal);
            var filePath = Path.Combine(path, filename);
            try
            {

                var fileStream = File.Open(filePath, FileMode.Create);
                stream.Position = 0;
                stream.CopyTo(fileStream);
                fileStream.Flush();
                fileStream.Close();
            }
            catch (Exception e)
            {

            }

            var currentController = UIApplication.SharedApplication.KeyWindow.RootViewController;
            while (currentController.PresentedViewController != null)
                currentController = currentController.PresentedViewController;
            var currentView = currentController.View;

            var qlPreview = new QLPreviewController();
            QLPreviewItem item = new QLPreviewItemBundle(filename, filePath);
            qlPreview.DataSource = new PreviewControllerDS(item);

            currentController.PresentViewController(qlPreview, true, null);
        }

    }

    public class QLPreviewItemBundle : QLPreviewItem
    {
        string _fileName, _filePath;
        public QLPreviewItemBundle(string fileName, string filePath)
        {
            _fileName = fileName;
            _filePath = filePath;
        }

        public override string ItemTitle
        {
            get
            {
                return _fileName;
            }
        }
        public override NSUrl ItemUrl
        {
            get
            {
                var documents = NSBundle.MainBundle.BundlePath;
                var lib = Path.Combine(documents, _filePath);
                var url = NSUrl.FromFilename(lib);
                return url;
            }
        }
    }

    public class PreviewControllerDS : QLPreviewControllerDataSource
    {
        private QLPreviewItem _item;

        public PreviewControllerDS(QLPreviewItem item)
        {
            _item = item;
        }

        public override nint PreviewItemCount(QLPreviewController controller)
        {
            return 1;
        }

        public override IQLPreviewItem GetPreviewItem(QLPreviewController controller, nint index)
        {
            return _item;
        }
    }
}